/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <gnome.h>
#include <gtk/gtk.h>
#include <libgnomeui/gnome-stock.h>
#include "pan.h"

static Props_window *props_window = NULL;
static GSList *servers;

/* Items that we will keep in memory, since
 * they might be looked up frequently */
static char *temp_dir = NULL;
static char *data_dir = NULL;
static char *download_dir = NULL;
static char *log_dir = NULL;

/*--------------------------------------------------------------------
 * functions for retrieving items that are kept in memory 
 *--------------------------------------------------------------------*/
inline char* props_get_temp_dir (void) { return temp_dir; }
inline char* props_get_data_dir (void) { return data_dir; }
inline char* props_get_download_dir (void) { return download_dir; }
inline char* props_get_log_dir (void) { return log_dir; };

/*--------------------------------------------------------------------
 * builds the server GSList from the data in the config files
 *--------------------------------------------------------------------*/
void
load_servers (void)
{
	char **sd;
	char *p;
	int i;
	server_data *sdata;
	char buf[256];

	if (servers) {
		g_slist_foreach(servers, (GFunc) server_free, NULL);
		g_slist_free(servers);
		servers = NULL;
	}
	if ((p = gnome_config_get_string("/Pan/Servers/Names")) == NULL)
		return;

	sd = g_strsplit(p, ",", 0);
	g_free(p);
	if (sd == NULL) {
		g_error("Error in config file.");
		return;
	}
	for (i = 0; sd[i]; i++) {
		g_snprintf(buf, 256, "/Pan/%s/", sd[i]);
		if (gnome_config_has_section(buf)) {
			gnome_config_push_prefix(buf);
			sdata = (server_data *) g_malloc0(sizeof(server_data));
			if ((sdata->address = gnome_config_get_string("Address")) ==
			    NULL) {
				g_free(sdata);
				break;
			}
			sdata->name = g_strdup(sd[i]);
			sdata->port =
				gnome_config_get_string_with_default("Port=119", NULL);
			sdata->username = gnome_config_get_string("Username");
			sdata->password = gnome_config_get_string("Password");
			servers = g_slist_append(servers, sdata);
			gnome_config_pop_prefix();
		}
	}
	g_strfreev(sd);
}

void 
save_servers (void)
{
	server_data *sdata = NULL;
	GSList *l = NULL;
	int i = 0;
	char *p = NULL;
	char **sd = NULL;
	char buf[256];

	/* Out with the old */
	if ((p = gnome_config_get_string("/Pan/Servers/Names"))) {
		sd = g_strsplit(p, ",", -1);
		for (i = 0; sd[i]; i++) {
			g_snprintf(buf, 256, "/Pan/%s", sd[i]);
			gnome_config_clean_section(buf);
		}
		g_strfreev(sd);
		g_free(p);
	}
	gnome_config_sync();

	/* In with the new */
	p = NULL;
	for (l = servers; l; l = l->next) {
		sdata = (server_data *) l->data;

		g_snprintf(buf, 256, "/Pan/%s/Address", sdata->name);
		gnome_config_set_string(buf, sdata->address);
		g_snprintf(buf, 256, "/Pan/%s/Port", sdata->name);
		gnome_config_set_string(buf, sdata->port);
		g_snprintf(buf, 256, "/Pan/%s/Username", sdata->name);
		gnome_config_set_string(buf, sdata->username);
		g_snprintf(buf, 256, "/Pan/%s/Password", sdata->name);
		gnome_config_set_string(buf, sdata->password);

		if (p) {
			p = g_realloc(p, strlen(p) + strlen(sdata->name) + 2);
			p = strcat(p, ",");
			p = strcat(p, sdata->name);
		} else
			p = g_strdup(sdata->name);
	}

	gnome_config_set_string("/Pan/Servers/Names", p);
	g_free(p);

	gnome_config_sync();
}


void props_init(void)
{
	/* Put code here that needs to be run
	 * whenever the app is started, or the
	 * properties are changed */

	/* Reset the cache of directory names */
	g_free(temp_dir);
	temp_dir = gnome_config_get_string("/Pan/Paths/temp_dir");
	if (temp_dir == NULL)
		temp_dir = gnome_util_prepend_user_home(".pan/tmp");
	directory_check(temp_dir);

	g_free(data_dir);
	data_dir = gnome_config_get_string("/Pan/Paths/data_dir");
	if (data_dir == NULL)
		data_dir = gnome_util_prepend_user_home(".pan/data");
	directory_check(data_dir);

	g_free(download_dir);
	download_dir = gnome_config_get_string("/Pan/Paths/download_dir");
	if (download_dir == NULL)
		download_dir = gnome_util_prepend_user_home(".pan/download");
	directory_check(download_dir);

	g_free(log_dir);
	log_dir = gnome_config_get_string("/Pan/Paths/log_dir");
	if (log_dir == NULL)
		log_dir = gnome_util_prepend_user_home(".pan/log");
	directory_check(log_dir);
}


void props_update_clist(void)
{
	char *text[2];
	GSList *l;
	server_data *sdata;
	int row;

	gtk_clist_freeze(GTK_CLIST(props_window->server_clist));
	gtk_clist_clear(GTK_CLIST(props_window->server_clist));

	for (l = servers; l; l = l->next) {
		sdata = (server_data *) l->data;
		text[0] = (sdata->name) ? sdata->name : _("Not Named");
		text[1] = (sdata->address) ? sdata->address : _("No Address");
		row =
			gtk_clist_prepend(GTK_CLIST(props_window->server_clist), text);
		gtk_clist_set_row_data(GTK_CLIST(props_window->server_clist), row,
				       sdata);
	}
	gtk_clist_thaw(GTK_CLIST(props_window->server_clist));
}

void props_apply(GtkWidget * box, int page_num)
{
	/* -1 means all pages, ignore anything else */
	if (page_num == -1) {
		/* Commit all of the data to the config file */

		/* No entry for these yet
		 * gnome_config_set_int("/Pan/Articles/Expiration", 30);
		 */

		gnome_config_set_string("/Pan/User/Full_Name",
					gtk_entry_get_text(GTK_ENTRY
							   (props_window->
							    fullname_entry)));
		gnome_config_set_string("/Pan/User/Email",
					gtk_entry_get_text(GTK_ENTRY
							   (props_window->
							    email_entry)));
		gnome_config_set_string("/Pan/User/Organization",
					gtk_entry_get_text(GTK_ENTRY
							   (props_window->
							    organization_entry)));
		gnome_config_set_string("/Pan/Paths/download_dir",
					gtk_entry_get_text(GTK_ENTRY
							   (props_window->
							    download_entry)));
		gnome_config_set_string("/Pan/Paths/temp_dir",
					gtk_entry_get_text(GTK_ENTRY
							   (props_window->
							    temp_entry)));
		gnome_config_set_string("/Pan/Paths/data_dir",
					gtk_entry_get_text(GTK_ENTRY
							   (props_window->
							    data_entry)));
		gnome_config_set_string("/Pan/Paths/log_dir",
					gtk_entry_get_text(GTK_ENTRY
							   (props_window->
							    log_entry)));
	
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(props_window->headers_0)))
			gnome_config_set_int("/Pan/Message/Headers", 0);
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(props_window->headers_1)))
			gnome_config_set_int("/Pan/Message/Headers", 1);
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(props_window->headers_2)))
			gnome_config_set_int("/Pan/Message/Headers", 2);
		
		save_servers();
		
		/* update_props_clist(); */
		gnome_config_sync();
		server_menu_update();
		props_init();
	}
}

void edit_profile_dialog_clicked(GnomeDialog * dialog, int button, server_data * data)
{

	switch (button) {
	case 0:					/* OK button */
		if (data == NULL) {
			data = (server_data *) g_malloc0(sizeof(server_data));
			servers = g_slist_append(servers, data);
		}
		data->name =
			g_strdup(gtk_entry_get_text
				 (GTK_ENTRY(props_window->profile_name_entry)));
		data->address =
			g_strdup(gtk_entry_get_text
				 (GTK_ENTRY(props_window->address_entry)));
		data->port =
			g_strdup(gtk_entry_get_text
				 (GTK_ENTRY(props_window->port_entry)));
		data->username =
			g_strdup(gtk_entry_get_text
				 (GTK_ENTRY(props_window->username_entry)));
		data->password =
			g_strdup(gtk_entry_get_text
				 (GTK_ENTRY(props_window->password_entry)));
		props_update_clist();
		gnome_property_box_changed(props_window->box);
		gtk_widget_destroy(GTK_WIDGET(dialog));
		break;
	default:					/* Cancel Button */
		gtk_widget_destroy(GTK_WIDGET(dialog));
		break;
	}
}


void edit_profile_changed(GnomeDialog * d)
{
	gnome_dialog_set_sensitive(d, 0, TRUE);
}


void edit_profile_dialog(server_data * data)
{
	GtkWidget *dialog;
	GtkWidget *table;
	GtkWidget *label;

	dialog = gnome_dialog_new("New/Edit Server", GNOME_STOCK_BUTTON_OK,
				  GNOME_STOCK_BUTTON_CANCEL, NULL);

	gnome_dialog_set_sensitive(GNOME_DIALOG(dialog), 0, FALSE);

	gtk_signal_connect(GTK_OBJECT(dialog), "clicked",
			   (GtkSignalFunc) edit_profile_dialog_clicked, data);

	table = gtk_table_new(4, 2, FALSE);
	gtk_widget_show(table);

	gtk_container_add(GTK_CONTAINER(GNOME_DIALOG(dialog)->vbox), table);

	label = gtk_label_new("Enter the Following Information:");
	gtk_widget_show(label);
	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 2, 0, 1);

/* Profile Name */
	label = gtk_label_new("Profile Name");
	gtk_widget_show(label);
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
			 GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);

	props_window->profile_name_entry = gtk_entry_new_with_max_length(32);
	if (data)
		gtk_entry_set_text(GTK_ENTRY(props_window->profile_name_entry),
				   data->name);
	gtk_signal_connect_object(GTK_OBJECT(props_window->profile_name_entry),
				  "changed",
				  GTK_SIGNAL_FUNC(edit_profile_changed),
				  GTK_OBJECT(dialog));
	gtk_widget_show(props_window->profile_name_entry);
	gtk_table_attach(GTK_TABLE(table), props_window->profile_name_entry, 1,
			 2, 1, 2, GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND,
			 20, 5);

/* Server Address */
	label = gtk_label_new("Server Address");
	gtk_widget_show(label);
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
			 GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);

	props_window->address_entry = gtk_entry_new_with_max_length(32);
	if (data)
		gtk_entry_set_text(GTK_ENTRY(props_window->address_entry),
				   data->address);
	gtk_signal_connect_object(GTK_OBJECT(props_window->address_entry),
				  "changed",
				  GTK_SIGNAL_FUNC(edit_profile_changed),
				  GTK_OBJECT(dialog));
	gtk_widget_show(props_window->address_entry);
	gtk_table_attach(GTK_TABLE(table), props_window->address_entry, 1, 2,
			 2, 3, GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND,
			 20, 5);

/* Server Port */
	label = gtk_label_new("Server Port");
	gtk_widget_show(label);
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4,
			 GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);

	props_window->port_entry = gtk_entry_new_with_max_length(32);
	gtk_entry_set_text(GTK_ENTRY(props_window->port_entry),
			   data ? data->port : "119");
	gtk_signal_connect_object(GTK_OBJECT(props_window->port_entry),
				  "changed",
				  GTK_SIGNAL_FUNC(edit_profile_changed),
				  GTK_OBJECT(dialog));
	gtk_widget_show(props_window->port_entry);
	gtk_table_attach(GTK_TABLE(table), props_window->port_entry, 1, 2, 3,
			 4, GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 20,
			 5);

/* Server Username */
	label = gtk_label_new("Username");
	gtk_widget_show(label);
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 4, 5,
			 GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);

	props_window->username_entry = gtk_entry_new_with_max_length(32);
	if (data && data->username)
		gtk_entry_set_text(GTK_ENTRY(props_window->username_entry),
				   data->username);
	gtk_signal_connect_object(GTK_OBJECT(props_window->username_entry),
				  "changed",
				  GTK_SIGNAL_FUNC(edit_profile_changed),
				  GTK_OBJECT(dialog));
	gtk_widget_show(props_window->username_entry);
	gtk_table_attach(GTK_TABLE(table), props_window->username_entry, 1, 2,
			 4, 5, GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND,
			 20, 5);

/* Server Password */
	label = gtk_label_new("Password");
	gtk_widget_show(label);
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 5, 6,
			 GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);

	props_window->password_entry = gtk_entry_new_with_max_length(32);
	gtk_entry_set_visibility(GTK_ENTRY(props_window->password_entry),
				 FALSE);
	if (data && data->password)
		gtk_entry_set_text(GTK_ENTRY(props_window->password_entry),
				   data->password);
	gtk_signal_connect_object(GTK_OBJECT(props_window->password_entry),
				  "changed",
				  GTK_SIGNAL_FUNC(edit_profile_changed),
				  GTK_OBJECT(dialog));
	gtk_widget_show(props_window->password_entry);
	gtk_table_attach(GTK_TABLE(table), props_window->password_entry, 1, 2,
			 5, 6, GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND,
			 20, 5);


	gtk_widget_show(dialog);
}

void server_clist_button_press(GtkWidget * widget, GdkEventButton * bevent)
{
	int row;
	int column;
	server_data *data;

	if (bevent->button != 1)
		gtk_signal_emit_stop_by_name(GTK_OBJECT(widget),
					     "button_press_event");

	if ((bevent->button) == 1 && (bevent->type == GDK_2BUTTON_PRESS)) {
		gtk_signal_emit_stop_by_name(GTK_OBJECT(widget),
					     "button_press_event");
		gtk_clist_get_selection_info(GTK_CLIST(widget), bevent->x,
					     bevent->y, &row, &column);
		data =
			(server_data *) gtk_clist_get_row_data(GTK_CLIST(widget), row);
		edit_profile_dialog(data);
	}
}

void new_server_cb(void)
{
	edit_profile_dialog(NULL);
}

void edit_server_cb(void)
{
	GList *list;
	server_data *data;

	if ((list = GTK_CLIST(props_window->server_clist)->selection) == NULL)
		return;
	data =
		(server_data *)
		gtk_clist_get_row_data(GTK_CLIST(props_window->server_clist),
				       GPOINTER_TO_INT(list->data));
	edit_profile_dialog(data);
}

void delete_server_cb(void)
{
	GList *l;
	server_data *data;

	if ((l = GTK_CLIST(props_window->server_clist)->selection) == NULL)
		return;
	data =
		(server_data *)
		gtk_clist_get_row_data(GTK_CLIST(props_window->server_clist),
				       GPOINTER_TO_INT(l->data));
	servers = g_slist_remove(servers, data);
	g_free(data);

	props_update_clist();
	gnome_property_box_changed(props_window->box);
}

void
props_cancel (void)
{
	gtk_widget_destroy(GTK_WIDGET(props_window->box));
	g_free(props_window);
	props_window = NULL;
	g_slist_foreach(servers, (GFunc) server_free, NULL);
	g_slist_free(servers);
	servers = NULL;
}

GtkWidget *
paths_page (void)
{
	GtkWidget *table;
	GtkWidget *label;
	char *p;

	table = gtk_table_new(4, 2, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(table), 2);
	gtk_table_set_col_spacings(GTK_TABLE(table), 2);

	label = gtk_label_new("Download Directory:");
	gtk_widget_show(label);
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
			 GTK_FILL, GTK_FILL, 4, 0);

	props_window->download_entry = gtk_entry_new();
	if ((p = gnome_config_get_string("/Pan/Paths/download_dir")) == NULL)
		p = gnome_util_prepend_user_home(".pan/download");

	gtk_entry_set_text(GTK_ENTRY(props_window->download_entry), p);
	g_free(p);
	gtk_signal_connect_object(GTK_OBJECT(props_window->download_entry),
				  "changed",
				  GTK_SIGNAL_FUNC(gnome_property_box_changed),
				  GTK_OBJECT(props_window->box));
	gtk_widget_show(props_window->download_entry);
	gtk_table_attach(GTK_TABLE(table), props_window->download_entry, 1, 2,
			 0, 1, GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

	label = gtk_label_new("Temporary Directory:");
	gtk_widget_show(label);
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
			 GTK_FILL, GTK_FILL, 4, 0);

	props_window->temp_entry = gtk_entry_new();
	if ((p = gnome_config_get_string("/Pan/Paths/temp_dir")) == NULL) {
		p = gnome_util_prepend_user_home(".pan/tmp");
	}
	gtk_entry_set_text(GTK_ENTRY(props_window->temp_entry), p);
	g_free(p);
	gtk_signal_connect_object(GTK_OBJECT(props_window->temp_entry),
				  "changed",
				  GTK_SIGNAL_FUNC(gnome_property_box_changed),
				  GTK_OBJECT(props_window->box));
	gtk_widget_show(props_window->temp_entry);
	gtk_table_attach(GTK_TABLE(table), props_window->temp_entry, 1, 2, 1,
			 2, GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

// Data Directory:
	label = gtk_label_new("Data Directory:");
	gtk_widget_show(label);
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
			 GTK_FILL, GTK_FILL, 4, 0);

	props_window->data_entry = gtk_entry_new();
	if ((p = gnome_config_get_string("/Pan/Paths/data_dir")) == NULL) {
		p = gnome_util_prepend_user_home(".pan/data");
	}
	gtk_entry_set_text(GTK_ENTRY(props_window->data_entry), p);
	g_free(p);
	gtk_signal_connect_object(GTK_OBJECT(props_window->data_entry),
				  "changed",
				  GTK_SIGNAL_FUNC(gnome_property_box_changed),
				  GTK_OBJECT(props_window->box));
	gtk_widget_show(props_window->data_entry);
	gtk_table_attach(GTK_TABLE(table), props_window->data_entry, 1, 2, 2,
			 3, GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

// Log File Directory:
	label = gtk_label_new("Log File Directory:");
	gtk_widget_show(label);
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4,
			 GTK_FILL, GTK_FILL, 4, 0);

	props_window->log_entry = gtk_entry_new();
	if ((p = gnome_config_get_string("/Pan/Paths/log_dir")) == NULL)
		p = gnome_util_prepend_user_home(".pan/log");
	gtk_entry_set_text(GTK_ENTRY(props_window->log_entry), p);
	g_free(p);
	gtk_signal_connect_object(GTK_OBJECT(props_window->log_entry),
				  "changed",
				  GTK_SIGNAL_FUNC(gnome_property_box_changed),
				  GTK_OBJECT(props_window->box));
	gtk_widget_show(props_window->log_entry);
	gtk_table_attach(GTK_TABLE(table), props_window->log_entry, 1, 2, 3, 4,
			 GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

	return table;
}

GtkWidget *
userinfo_page (void)
{
	GtkWidget *table;
	GtkWidget *label;
	char *p;

	table = gtk_table_new(3, 2, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(table), 2);
	gtk_table_set_col_spacings(GTK_TABLE(table), 2);

	label = gtk_label_new("Full Name:");
	gtk_widget_show(label);
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
			 GTK_FILL, GTK_FILL, 4, 0);

	props_window->fullname_entry = gtk_entry_new();
	p = gnome_config_get_string("/Pan/User/Full_Name");
	gtk_entry_set_text(GTK_ENTRY(props_window->fullname_entry), p);
	g_free(p);
	gtk_signal_connect_object(GTK_OBJECT(props_window->fullname_entry),
				  "changed",
				  GTK_SIGNAL_FUNC(gnome_property_box_changed),
				  GTK_OBJECT(props_window->box));
	gtk_widget_show(props_window->fullname_entry);
	gtk_table_attach(GTK_TABLE(table), props_window->fullname_entry, 1, 2,
			 0, 1, GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

	label = gtk_label_new("Email Address:");
	gtk_widget_show(label);
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
			 GTK_FILL, GTK_FILL, 4, 0);

	props_window->email_entry = gtk_entry_new();
	p = gnome_config_get_string("/Pan/User/Email");
	gtk_entry_set_text(GTK_ENTRY(props_window->email_entry), p);
	g_free(p);
	gtk_signal_connect_object(GTK_OBJECT(props_window->email_entry),
				  "changed",
				  GTK_SIGNAL_FUNC(gnome_property_box_changed),
				  GTK_OBJECT(props_window->box));
	gtk_widget_show(props_window->email_entry);
	gtk_table_attach(GTK_TABLE(table), props_window->email_entry, 1, 2, 1,
			 2, GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

	label = gtk_label_new("Organization:");
	gtk_widget_show(label);
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
			 GTK_FILL, GTK_FILL, 4, 0);

	props_window->organization_entry = gtk_entry_new();
	p = gnome_config_get_string("/Pan/User/Organization");
	gtk_entry_set_text(GTK_ENTRY(props_window->organization_entry), p);
	g_free(p);
	gtk_signal_connect_object(GTK_OBJECT(props_window->organization_entry),
				  "changed",
				  GTK_SIGNAL_FUNC(gnome_property_box_changed),
				  GTK_OBJECT(props_window->box));
	gtk_widget_show(props_window->organization_entry);
	gtk_table_attach(GTK_TABLE(table), props_window->organization_entry, 1,
			 2, 2, 3, GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

	return table;
}

GtkWidget *
message_page (void)
{
	GtkWidget *table;
	GtkWidget *w;
	char *p;
	int i;
	
	table = gtk_table_new(3, 2, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(table), 2);
	gtk_table_set_col_spacings(GTK_TABLE(table), 2);

	i = gnome_config_get_int("/Pan/Message/Headers=1"); 
	
	w = gtk_label_new("Header style: ");
	gtk_table_attach(GTK_TABLE(table), w, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 4, 0);
    
	props_window->headers_0 = gtk_radio_button_new_with_label(NULL, _("Brief"));
	gtk_table_attach(GTK_TABLE(table), props_window->headers_0, 1, 2, 0, 1, GTK_FILL, GTK_FILL, 4, 0);
	if (i == 0)
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(props_window->headers_0), TRUE);
	gtk_signal_connect_object(GTK_OBJECT(props_window->headers_0), "clicked",
				  GTK_SIGNAL_FUNC(gnome_property_box_changed),
				  GTK_OBJECT(props_window->box));

	props_window->headers_1 = gtk_radio_button_new_with_label(
		gtk_radio_button_group(GTK_RADIO_BUTTON(props_window->headers_0)), _("Partial"));
	gtk_table_attach(GTK_TABLE(table), props_window->headers_1, 1, 2, 1, 2, GTK_FILL, GTK_FILL, 4, 0);
	if (i == 1)
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(props_window->headers_1), TRUE);
	gtk_signal_connect_object(GTK_OBJECT(props_window->headers_1), "clicked",
				  GTK_SIGNAL_FUNC(gnome_property_box_changed),
				  GTK_OBJECT(props_window->box));
	
	props_window->headers_2 = gtk_radio_button_new_with_label(
		gtk_radio_button_group(GTK_RADIO_BUTTON(props_window->headers_1)), _("Full"));
	gtk_table_attach(GTK_TABLE(table), props_window->headers_2, 1, 2, 2, 3, GTK_FILL, GTK_FILL, 4, 0);
	if (i == 2)
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(props_window->headers_2), TRUE);
	gtk_signal_connect_object(GTK_OBJECT(props_window->headers_2), "clicked",
				  GTK_SIGNAL_FUNC(gnome_property_box_changed),
				  GTK_OBJECT(props_window->box));

	gtk_widget_show_all(GTK_WIDGET(table));
	return table;
}

GtkWidget *
server_page (void)
{
	GtkWidget *vbox, *hbox, *button;

	static char *titles[] = {
		_("Name"),
		_("Address"),
	};

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_widget_show(hbox);

	props_window->server_clist = gtk_clist_new_with_titles(2, titles);
	gtk_clist_column_titles_passive(GTK_CLIST(props_window->server_clist));
	gtk_clist_set_selection_mode(GTK_CLIST(props_window->server_clist),
				     GTK_SELECTION_SINGLE);

	gtk_clist_set_column_width(GTK_CLIST(props_window->server_clist), 0, 200);
	gtk_clist_set_column_width(GTK_CLIST(props_window->server_clist), 1, 40);

	gtk_widget_set_usize(props_window->server_clist, 280, 200);
	gtk_box_pack_start(GTK_BOX(hbox), props_window->server_clist, TRUE,
			   TRUE, 5);
	gtk_signal_connect(GTK_OBJECT(props_window->server_clist),
			   "button_press_event",
			   GTK_SIGNAL_FUNC(server_clist_button_press), NULL);

	gtk_widget_show(props_window->server_clist);
	props_update_clist();

	/* one button vbox */
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 5);
	gtk_widget_show(vbox);

	/* new server button */
	button = gtk_button_new_with_label("New...");
	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 10);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
			   (GtkSignalFunc) new_server_cb, NULL);
	gtk_widget_show(button);

	/* edit server button */
	button = gtk_button_new_with_label("Edit...");
	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 10);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
			   (GtkSignalFunc) edit_server_cb, NULL);
	gtk_widget_show(button);

	/* delete server button */
	button = gtk_button_new_with_label("Delete");
	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 10);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
			   (GtkSignalFunc) delete_server_cb, NULL);
	gtk_widget_show(button);

	return hbox;
}

void
props_spawn (void)
{
	GtkWidget *w, *label;

	/* There can be only one! (properties window) */
	if (props_window) {
		gdk_window_raise(GTK_WIDGET(props_window->box)->window);
		return;
	}
	load_servers();

	props_window = (Props_window *) g_malloc(sizeof(Props_window));
	props_window->box = GNOME_PROPERTY_BOX(gnome_property_box_new());
	w = server_page();
	gtk_widget_show(w);
	label = gtk_label_new(_("Servers"));
	gtk_widget_show(label);
	gtk_notebook_append_page(GTK_NOTEBOOK(props_window->box->notebook), w,
				 label);
	w = userinfo_page();
	gtk_widget_show(w);
	label = gtk_label_new(_("User Info"));
	gtk_widget_show(label);
	gtk_notebook_append_page(GTK_NOTEBOOK(props_window->box->notebook), w,
				 label);
	w = paths_page();
	gtk_widget_show(w);
	label = gtk_label_new(_("Paths"));
	gtk_widget_show(label);
	gtk_notebook_append_page(GTK_NOTEBOOK(props_window->box->notebook), w,
				 label);
	w = message_page();
	gtk_widget_show(w);
	label = gtk_label_new(_("Message"));
	gtk_widget_show(label);
	gtk_notebook_append_page(GTK_NOTEBOOK(props_window->box->notebook), w, label);
	
	gtk_signal_connect(GTK_OBJECT(props_window->box), "apply",
			   GTK_SIGNAL_FUNC(props_apply), NULL);
	gtk_signal_connect(GTK_OBJECT(props_window->box), "destroy",
			   GTK_SIGNAL_FUNC(props_cancel), NULL);
	gtk_widget_show(GTK_WIDGET(props_window->box));
}
